declare global {
  interface Window {
    PREFIX_DISABLED: boolean
  }
}

/**
 * Pushes an index to an associated key in the local storage.
 *
 * @param {string} index - The index to be pushed
 * @param {string} key - The associated key
 */
const pushToAssoc = (index: string, key: string) => {
  const assocKey = `$assoc#${key}`
  const assoc = JSON.parse(localStorage.getItem(assocKey) || '[]') as string[]

  if (!assoc.includes(index)) {
    assoc.push(index)
  }
  localStorage.setItem(assocKey, JSON.stringify(assoc))
}

/**
 * Generates a key based on the provided index and optional key.
 *
 * @param {string} index - The index value for the key
 * @param {string} [_key] - An optional value setting the context for the key
 * @returns {Promise<string>} A promise that resolves to the generated key
 *
 * If PREFIX_DISABLED is set in the window object, the function will return
 * "!{index}".
 *
 * If a _key is provided, or there is a 'key' item in the localStorage,
 * it will be used in the generation of the key. If neither is available,
 * '!default' will be used.
 *
 * The generated key is then pushed to an associative array.
 */
const getKey = async (index: string, _key?: string) => {
  if (window.PREFIX_DISABLED) {
    return `!${index}`
  }
  const key = _key || localStorage.getItem('key') || '!default'
  const hashKey = `@${key}:${index}`
  pushToAssoc(hashKey, key)
  return hashKey
}

export default getKey
